# mypy: allow-untyped-defs
from __future__ import annotations

import sys
import warnings

import pytest
from pytest import ExitCode
from pytest import Pytester
from pytest import WarningsRecorder


def test_recwarn_stacklevel(recwarn: WarningsRecorder) -> None:
    warnings.warn("hello")
    warn = recwarn.pop()
    assert warn.filename == __file__


def test_recwarn_functional(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import warnings
        def test_method(recwarn):
            warnings.warn("hello")
            warn = recwarn.pop()
            assert isinstance(warn.message, UserWarning)
    """
    )
    reprec = pytester.inline_run()
    reprec.assertoutcome(passed=1)


@pytest.mark.filterwarnings("")
def test_recwarn_captures_deprecation_warning(recwarn: WarningsRecorder) -> None:
    """
    Check that recwarn can capture DeprecationWarning by default
    without custom filterwarnings (see #8666).
    """
    warnings.warn(DeprecationWarning("some deprecation"))
    assert len(recwarn) == 1
    assert recwarn.pop(DeprecationWarning)


class TestSubclassWarningPop:
    class ParentWarning(Warning):
        pass

    class ChildWarning(ParentWarning):
        pass

    class ChildOfChildWarning(ChildWarning):
        pass

    @staticmethod
    def raise_warnings_from_list(_warnings: list[type[Warning]]):
        for warn in _warnings:
            warnings.warn(f"Warning {warn().__repr__()}", warn)

    def test_pop_finds_exact_match(self):
        with pytest.warns((self.ParentWarning, self.ChildWarning)) as record:
            self.raise_warnings_from_list(
                [self.ChildWarning, self.ParentWarning, self.ChildOfChildWarning]
            )

        assert len(record) == 3
        _warn = record.pop(self.ParentWarning)
        assert _warn.category is self.ParentWarning

    def test_pop_raises_if_no_match(self):
        with pytest.raises(AssertionError):
            with pytest.warns(self.ParentWarning) as record:
                self.raise_warnings_from_list([self.ParentWarning])
            record.pop(self.ChildOfChildWarning)

    def test_pop_finds_best_inexact_match(self):
        with pytest.warns(self.ParentWarning) as record:
            self.raise_warnings_from_list(
                [self.ChildOfChildWarning, self.ChildWarning, self.ChildOfChildWarning]
            )

        _warn = record.pop(self.ParentWarning)
        assert _warn.category is self.ChildWarning


class TestWarningsRecorderChecker:
    def test_recording(self) -> None:
        rec = WarningsRecorder(_ispytest=True)
        with rec:
            assert not rec.list
            warnings.warn_explicit("hello", UserWarning, "xyz", 13)
            assert len(rec.list) == 1
            warnings.warn(DeprecationWarning("hello"))
            assert len(rec.list) == 2
            warn = rec.pop()
            assert str(warn.message) == "hello"
            values = rec.list
            rec.clear()
            assert len(rec.list) == 0
            assert values is rec.list
            with pytest.raises(AssertionError):
                rec.pop()

    def test_warn_stacklevel(self) -> None:
        """#4243"""
        rec = WarningsRecorder(_ispytest=True)
        with rec:
            warnings.warn("test", DeprecationWarning, 2)

    def test_typechecking(self) -> None:
        from _pytest.recwarn import WarningsChecker

        with pytest.raises(TypeError):
            WarningsChecker(5, _ispytest=True)  # type: ignore[arg-type]
        with pytest.raises(TypeError):
            WarningsChecker(("hi", RuntimeWarning), _ispytest=True)  # type: ignore[arg-type]
        with pytest.raises(TypeError):
            WarningsChecker([DeprecationWarning, RuntimeWarning], _ispytest=True)  # type: ignore[arg-type]

    def test_invalid_enter_exit(self) -> None:
        # wrap this test in WarningsRecorder to ensure warning state gets reset
        with WarningsRecorder(_ispytest=True):
            with pytest.raises(RuntimeError):
                rec = WarningsRecorder(_ispytest=True)
                rec.__exit__(None, None, None)  # can't exit before entering

            with pytest.raises(RuntimeError):
                rec = WarningsRecorder(_ispytest=True)
                with rec:
                    with rec:
                        pass  # can't enter twice


class TestDeprecatedCall:
    """test pytest.deprecated_call()"""

    def dep(self, i: int, j: int | None = None) -> int:
        if i == 0:
            warnings.warn("is deprecated", DeprecationWarning, stacklevel=1)
        return 42

    def dep_explicit(self, i: int) -> None:
        if i == 0:
            warnings.warn_explicit(
                "dep_explicit", category=DeprecationWarning, filename="hello", lineno=3
            )

    def test_deprecated_call_raises(self) -> None:
        with pytest.raises(pytest.fail.Exception, match="No warnings of type"):
            with pytest.deprecated_call():
                self.dep(3, 5)

    def test_deprecated_call(self) -> None:
        with pytest.deprecated_call():
            self.dep(0, 5)

    def test_deprecated_call_ret(self) -> None:
        ret = pytest.deprecated_call(self.dep, 0)
        assert ret == 42

    def test_deprecated_call_preserves(self) -> None:
        # Type ignored because `onceregistry` and `filters` are not
        # documented API.
        onceregistry = warnings.onceregistry.copy()  # type: ignore
        filters = warnings.filters[:]
        warn = warnings.warn
        warn_explicit = warnings.warn_explicit
        self.test_deprecated_call_raises()
        self.test_deprecated_call()
        assert onceregistry == warnings.onceregistry  # type: ignore
        assert filters == warnings.filters
        assert warn is warnings.warn
        assert warn_explicit is warnings.warn_explicit

    def test_deprecated_explicit_call_raises(self) -> None:
        with pytest.raises(pytest.fail.Exception):
            with pytest.deprecated_call():
                self.dep_explicit(3)

    def test_deprecated_explicit_call(self) -> None:
        with pytest.deprecated_call():
            self.dep_explicit(0)
        with pytest.deprecated_call():
            self.dep_explicit(0)

    @pytest.mark.parametrize("mode", ["context_manager", "call"])
    def test_deprecated_call_no_warning(self, mode) -> None:
        """Ensure deprecated_call() raises the expected failure when its block/function does
        not raise a deprecation warning.
        """

        def f():
            pass

        msg = "No warnings of type (.*DeprecationWarning.*, .*PendingDeprecationWarning.*)"
        with pytest.raises(pytest.fail.Exception, match=msg):
            if mode == "call":
                pytest.deprecated_call(f)
            else:
                with pytest.deprecated_call():
                    f()

    @pytest.mark.parametrize(
        "warning_type", [PendingDeprecationWarning, DeprecationWarning, FutureWarning]
    )
    @pytest.mark.parametrize("mode", ["context_manager", "call"])
    @pytest.mark.parametrize("call_f_first", [True, False])
    @pytest.mark.filterwarnings("ignore:hi")
    def test_deprecated_call_modes(self, warning_type, mode, call_f_first) -> None:
        """Ensure deprecated_call() captures a deprecation warning as expected inside its
        block/function.
        """

        def f():
            warnings.warn(warning_type("hi"))
            return 10

        # ensure deprecated_call() can capture the warning even if it has already been triggered
        if call_f_first:
            assert f() == 10
        if mode == "call":
            assert pytest.deprecated_call(f) == 10
        else:
            with pytest.deprecated_call():
                assert f() == 10

    def test_deprecated_call_specificity(self) -> None:
        other_warnings = [
            Warning,
            UserWarning,
            SyntaxWarning,
            RuntimeWarning,
            ImportWarning,
            UnicodeWarning,
        ]
        for warning in other_warnings:

            def f():
                warnings.warn(warning("hi"))  # noqa: B023

            with pytest.warns(warning):
                with pytest.raises(pytest.fail.Exception):
                    pytest.deprecated_call(f)
                with pytest.raises(pytest.fail.Exception):
                    with pytest.deprecated_call():
                        f()

    def test_deprecated_call_supports_match(self) -> None:
        with pytest.deprecated_call(match=r"must be \d+$"):
            warnings.warn("value must be 42", DeprecationWarning)

        with pytest.deprecated_call():
            with pytest.raises(
                pytest.fail.Exception, match="Regex pattern did not match"
            ):
                with pytest.deprecated_call(match=r"must be \d+$"):
                    warnings.warn("this is not here", DeprecationWarning)


class TestWarns:
    def test_check_callable(self) -> None:
        source = "warnings.warn('w1', RuntimeWarning)"
        with pytest.raises(TypeError, match=r".* must be callable"):
            pytest.warns(RuntimeWarning, source)  # type: ignore

    def test_several_messages(self) -> None:
        # different messages, b/c Python suppresses multiple identical warnings
        with pytest.warns(RuntimeWarning):
            warnings.warn("w1", RuntimeWarning)
        with pytest.warns(RuntimeWarning):
            with pytest.raises(pytest.fail.Exception):
                with pytest.warns(UserWarning):
                    warnings.warn("w2", RuntimeWarning)
        with pytest.warns(RuntimeWarning):
            warnings.warn("w3", RuntimeWarning)

    def test_function(self) -> None:
        pytest.warns(
            SyntaxWarning, lambda msg: warnings.warn(msg, SyntaxWarning), "syntax"
        )

    def test_warning_tuple(self) -> None:
        with pytest.warns((RuntimeWarning, SyntaxWarning)):
            warnings.warn("w1", RuntimeWarning)
        with pytest.warns((RuntimeWarning, SyntaxWarning)):
            warnings.warn("w2", SyntaxWarning)
        with pytest.warns(UserWarning, match="^w3$"):
            with pytest.raises(pytest.fail.Exception):
                with pytest.warns((RuntimeWarning, SyntaxWarning)):
                    warnings.warn("w3", UserWarning)

    def test_as_contextmanager(self) -> None:
        with pytest.warns(RuntimeWarning):
            warnings.warn("runtime", RuntimeWarning)

        with pytest.warns(UserWarning):
            warnings.warn("user", UserWarning)

        with pytest.warns():
            with pytest.raises(pytest.fail.Exception) as excinfo:
                with pytest.warns(RuntimeWarning):
                    warnings.warn("user", UserWarning)
        excinfo.match(
            r"DID NOT WARN. No warnings of type \(.+RuntimeWarning.+,\) were emitted.\n"
            r" Emitted warnings: \[UserWarning\('user',?\)\]."
        )

        with pytest.warns():
            with pytest.raises(pytest.fail.Exception) as excinfo:
                with pytest.warns(UserWarning):
                    warnings.warn("runtime", RuntimeWarning)
        excinfo.match(
            r"DID NOT WARN. No warnings of type \(.+UserWarning.+,\) were emitted.\n"
            r" Emitted warnings: \[RuntimeWarning\('runtime',?\)]."
        )

        with pytest.raises(pytest.fail.Exception) as excinfo:
            with pytest.warns(UserWarning):
                pass
        excinfo.match(
            r"DID NOT WARN. No warnings of type \(.+UserWarning.+,\) were emitted.\n"
            r" Emitted warnings: \[\]."
        )

        warning_classes = (UserWarning, FutureWarning)
        with pytest.warns():
            with pytest.raises(pytest.fail.Exception) as excinfo:
                with pytest.warns(warning_classes) as warninfo:
                    warnings.warn("runtime", RuntimeWarning)
                    warnings.warn("import", ImportWarning)

        messages = [each.message for each in warninfo]
        expected_str = (
            f"DID NOT WARN. No warnings of type {warning_classes} were emitted.\n"
            f" Emitted warnings: {messages}."
        )

        assert str(excinfo.value) == expected_str

    def test_record(self) -> None:
        with pytest.warns(UserWarning) as record:
            warnings.warn("user", UserWarning)

        assert len(record) == 1
        assert str(record[0].message) == "user"

    def test_record_only(self) -> None:
        with pytest.warns() as record:
            warnings.warn("user", UserWarning)
            warnings.warn("runtime", RuntimeWarning)

        assert len(record) == 2
        assert str(record[0].message) == "user"
        assert str(record[1].message) == "runtime"

    def test_record_only_none_type_error(self) -> None:
        with pytest.raises(TypeError):
            pytest.warns(None)  # type: ignore[call-overload]

    def test_record_by_subclass(self) -> None:
        with pytest.warns(Warning) as record:
            warnings.warn("user", UserWarning)
            warnings.warn("runtime", RuntimeWarning)

        assert len(record) == 2
        assert str(record[0].message) == "user"
        assert str(record[1].message) == "runtime"

        class MyUserWarning(UserWarning):
            pass

        class MyRuntimeWarning(RuntimeWarning):
            pass

        with pytest.warns((UserWarning, RuntimeWarning)) as record:
            warnings.warn("user", MyUserWarning)
            warnings.warn("runtime", MyRuntimeWarning)

        assert len(record) == 2
        assert str(record[0].message) == "user"
        assert str(record[1].message) == "runtime"

    def test_double_test(self, pytester: Pytester) -> None:
        """If a test is run again, the warning should still be raised"""
        pytester.makepyfile(
            """
            import pytest
            import warnings

            @pytest.mark.parametrize('run', [1, 2])
            def test(run):
                with pytest.warns(RuntimeWarning):
                    warnings.warn("runtime", RuntimeWarning)
        """
        )
        result = pytester.runpytest()
        result.stdout.fnmatch_lines(["*2 passed in*"])

    def test_match_regex(self) -> None:
        with pytest.warns(UserWarning, match=r"must be \d+$"):
            warnings.warn("value must be 42", UserWarning)

        with pytest.warns():
            with pytest.raises(pytest.fail.Exception):
                with pytest.warns(UserWarning, match=r"must be \d+$"):
                    warnings.warn("this is not here", UserWarning)

        with pytest.warns():
            with pytest.raises(pytest.fail.Exception):
                with pytest.warns(FutureWarning, match=r"must be \d+$"):
                    warnings.warn("value must be 42", UserWarning)

    def test_one_from_multiple_warns(self) -> None:
        with pytest.warns():
            with pytest.raises(
                pytest.fail.Exception, match="Regex pattern did not match"
            ):
                with pytest.warns(UserWarning, match=r"aaa"):
                    with pytest.warns(UserWarning, match=r"aaa"):
                        warnings.warn("cccccccccc", UserWarning)
                        warnings.warn("bbbbbbbbbb", UserWarning)
                        warnings.warn("aaaaaaaaaa", UserWarning)

    def test_none_of_multiple_warns(self) -> None:
        with pytest.warns():
            with pytest.raises(
                pytest.fail.Exception, match="Regex pattern did not match"
            ):
                with pytest.warns(UserWarning, match=r"aaa"):
                    warnings.warn("bbbbbbbbbb", UserWarning)
                    warnings.warn("cccccccccc", UserWarning)

    def test_warns_match_failure_message_detail(self) -> None:
        with pytest.warns():
            with pytest.raises(pytest.fail.Exception) as excinfo:
                with pytest.warns(UserWarning, match=r"must be \d+$"):
                    warnings.warn("this is not here", UserWarning)
        msg = str(excinfo.value)
        assert "Regex pattern did not match" in msg
        assert "this is not here" in msg
        assert "DID NOT WARN" not in msg

    def test_warns_match_re_escape_hint(self) -> None:
        with pytest.warns():
            with pytest.raises(pytest.fail.Exception) as excinfo:
                with pytest.warns(UserWarning, match="foo (bar)"):
                    warnings.warn("foo (bar)", UserWarning)
        assert "re.escape()" in str(excinfo.value)

    def test_warns_match_re_escape_hint_no_false_positive(self) -> None:
        with pytest.warns():
            with pytest.raises(pytest.fail.Exception) as excinfo:
                with pytest.warns(DeprecationWarning, match="foo (bar)"):
                    warnings.warn("some deprecation msg", DeprecationWarning)
                    warnings.warn("foo (bar)", UserWarning)
        assert "re.escape()" not in str(excinfo.value)

    @pytest.mark.filterwarnings("ignore")
    def test_can_capture_previously_warned(self) -> None:
        def f() -> int:
            warnings.warn(UserWarning("ohai"))
            return 10

        assert f() == 10
        assert pytest.warns(UserWarning, f) == 10
        assert pytest.warns(UserWarning, f) == 10
        assert pytest.warns(UserWarning, f) != "10"  # type: ignore[comparison-overlap]

    def test_warns_context_manager_with_kwargs(self) -> None:
        with pytest.raises(TypeError) as excinfo:
            with pytest.warns(UserWarning, foo="bar"):  # type: ignore
                pass
        assert "Unexpected keyword arguments" in str(excinfo.value)

    def test_re_emit_single(self) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.warns(UserWarning):
                warnings.warn("user warning", UserWarning)
                warnings.warn("some deprecation warning", DeprecationWarning)

    def test_re_emit_multiple(self) -> None:
        with pytest.warns(UserWarning):
            warnings.warn("first warning", UserWarning)
            warnings.warn("second warning", UserWarning)

    def test_re_emit_match_single(self) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.warns(UserWarning, match="user warning"):
                warnings.warn("user warning", UserWarning)
                warnings.warn("some deprecation warning", DeprecationWarning)

    def test_re_emit_match_multiple(self) -> None:
        with warnings.catch_warnings():
            warnings.simplefilter("error")  # if anything is re-emitted
            with pytest.warns(UserWarning, match="user warning"):
                warnings.warn("first user warning", UserWarning)
                warnings.warn("second user warning", UserWarning)

    def test_re_emit_non_match_single(self) -> None:
        with pytest.warns(UserWarning, match="v2 warning"):
            with pytest.warns(UserWarning, match="v1 warning"):
                warnings.warn("v1 warning", UserWarning)
                warnings.warn("non-matching v2 warning", UserWarning)

    def test_catch_warning_within_raise(self) -> None:
        # warns-in-raises works since https://github.com/pytest-dev/pytest/pull/11129
        with pytest.raises(ValueError, match="some exception"):
            with pytest.warns(FutureWarning, match="some warning"):
                warnings.warn("some warning", category=FutureWarning)
                raise ValueError("some exception")
        # and raises-in-warns has always worked but we'll check for symmetry.
        with pytest.warns(FutureWarning, match="some warning"):
            with pytest.raises(ValueError, match="some exception"):
                warnings.warn("some warning", category=FutureWarning)
                raise ValueError("some exception")

    def test_skip_within_warns(self, pytester: Pytester) -> None:
        """Regression test for #11907."""
        pytester.makepyfile(
            """
            import pytest

            def test_it():
                with pytest.warns(Warning):
                    pytest.skip("this is OK")
            """,
        )

        result = pytester.runpytest()
        assert result.ret == ExitCode.OK
        result.assert_outcomes(skipped=1)

    def test_fail_within_warns(self, pytester: Pytester) -> None:
        """Regression test for #11907."""
        pytester.makepyfile(
            """
            import pytest

            def test_it():
                with pytest.warns(Warning):
                    pytest.fail("BOOM")
            """,
        )

        result = pytester.runpytest()
        assert result.ret == ExitCode.TESTS_FAILED
        result.assert_outcomes(failed=1)
        assert "DID NOT WARN" not in str(result.stdout)

    def test_exit_within_warns(self, pytester: Pytester) -> None:
        """Regression test for #11907."""
        pytester.makepyfile(
            """
            import pytest

            def test_it():
                with pytest.warns(Warning):
                    pytest.exit()
            """,
        )

        result = pytester.runpytest()
        assert result.ret == ExitCode.INTERRUPTED
        result.assert_outcomes()

    def test_keyboard_interrupt_within_warns(self, pytester: Pytester) -> None:
        """Regression test for #11907."""
        pytester.makepyfile(
            """
            import pytest

            def test_it():
                with pytest.warns(Warning):
                    raise KeyboardInterrupt()
            """,
        )

        result = pytester.runpytest_subprocess()
        assert result.ret == ExitCode.INTERRUPTED
        result.assert_outcomes()


def test_raise_type_error_on_invalid_warning() -> None:
    """Check pytest.warns validates warning messages are strings (#10865) or
    Warning instances (#11959)."""
    with pytest.raises(TypeError, match="Warning must be str or Warning"):
        with pytest.warns(UserWarning):
            warnings.warn(1)  # type: ignore


@pytest.mark.parametrize(
    "message",
    [
        pytest.param("Warning", id="str"),
        pytest.param(UserWarning(), id="UserWarning"),
        pytest.param(Warning(), id="Warning"),
    ],
)
def test_no_raise_type_error_on_valid_warning(message: str | Warning) -> None:
    """Check pytest.warns validates warning messages are strings (#10865) or
    Warning instances (#11959)."""
    with pytest.warns(Warning):
        warnings.warn(message)


@pytest.mark.skipif(
    hasattr(sys, "pypy_version_info"),
    reason="Not for pypy",
)
def test_raise_type_error_on_invalid_warning_message_cpython() -> None:
    # Check that we get the same behavior with the stdlib, at least if filtering
    # (see https://github.com/python/cpython/issues/103577 for details)
    with pytest.raises(TypeError):
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", "test")
            warnings.warn(1)  # type: ignore


def test_multiple_arg_custom_warning() -> None:
    """Test for issue #11906."""

    class CustomWarning(UserWarning):
        def __init__(self, a, b):
            pass

    with pytest.warns(CustomWarning):
        with pytest.raises(pytest.fail.Exception, match="Regex pattern did not match"):
            with pytest.warns(CustomWarning, match="not gonna match"):
                a, b = 1, 2
                warnings.warn(CustomWarning(a, b))



from __future__ import annotations

import gc
import sys
from unittest import mock

from _pytest.pytester import Pytester
import pytest


PYPY = hasattr(sys, "pypy_version_info")

UNRAISABLE_LINE = (
    (
        "  * PytestUnraisableExceptionWarning: Exception ignored while calling "
        "deallocator <function BrokenDel.__del__ at *>: None"
    )
    if sys.version_info >= (3, 14)
    else "  * PytestUnraisableExceptionWarning: Exception ignored in: <function BrokenDel.__del__ at *>"
)

TRACEMALLOC_LINES = (
    ()
    if sys.version_info >= (3, 14)
    else (
        "  Enable tracemalloc to get traceback where the object was allocated.",
        "  See https* for more info.",
    )
)


@pytest.mark.skipif(PYPY, reason="garbage-collection differences make this flaky")
@pytest.mark.filterwarnings("default::pytest.PytestUnraisableExceptionWarning")
def test_unraisable(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it="""
        class BrokenDel:
            def __del__(self):
                raise ValueError("del is broken")

        def test_it():
            obj = BrokenDel()
            del obj

        def test_2(): pass
        """
    )
    result = pytester.runpytest()
    assert result.ret == 0
    result.assert_outcomes(passed=2, warnings=1)
    result.stdout.fnmatch_lines(
        [
            "*= warnings summary =*",
            "test_it.py::test_it",
            UNRAISABLE_LINE,
            "  ",
            "  Traceback (most recent call last):",
            "  ValueError: del is broken",
            "  ",
            *TRACEMALLOC_LINES,
            "    warnings.warn(pytest.PytestUnraisableExceptionWarning(msg))",
        ]
    )


@pytest.mark.skipif(PYPY, reason="garbage-collection differences make this flaky")
@pytest.mark.filterwarnings("default::pytest.PytestUnraisableExceptionWarning")
def test_unraisable_in_setup(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it="""
        import pytest

        class BrokenDel:
            def __del__(self):
                raise ValueError("del is broken")

        @pytest.fixture
        def broken_del():
            obj = BrokenDel()
            del obj

        def test_it(broken_del): pass
        def test_2(): pass
        """
    )
    result = pytester.runpytest()
    assert result.ret == 0
    result.assert_outcomes(passed=2, warnings=1)
    result.stdout.fnmatch_lines(
        [
            "*= warnings summary =*",
            "test_it.py::test_it",
            UNRAISABLE_LINE,
            "  ",
            "  Traceback (most recent call last):",
            "  ValueError: del is broken",
            "  ",
            *TRACEMALLOC_LINES,
            "    warnings.warn(pytest.PytestUnraisableExceptionWarning(msg))",
        ]
    )


@pytest.mark.skipif(PYPY, reason="garbage-collection differences make this flaky")
@pytest.mark.filterwarnings("default::pytest.PytestUnraisableExceptionWarning")
def test_unraisable_in_teardown(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it="""
        import pytest

        class BrokenDel:
            def __del__(self):
                raise ValueError("del is broken")

        @pytest.fixture
        def broken_del():
            yield
            obj = BrokenDel()
            del obj

        def test_it(broken_del): pass
        def test_2(): pass
        """
    )
    result = pytester.runpytest()
    assert result.ret == 0
    result.assert_outcomes(passed=2, warnings=1)
    result.stdout.fnmatch_lines(
        [
            "*= warnings summary =*",
            "test_it.py::test_it",
            UNRAISABLE_LINE,
            "  ",
            "  Traceback (most recent call last):",
            "  ValueError: del is broken",
            "  ",
            *TRACEMALLOC_LINES,
            "    warnings.warn(pytest.PytestUnraisableExceptionWarning(msg))",
        ]
    )


@pytest.mark.filterwarnings("error::pytest.PytestUnraisableExceptionWarning")
def test_unraisable_warning_error(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it=f"""
        class BrokenDel:
            def __del__(self) -> None:
                raise ValueError("del is broken")

        def test_it() -> None:
            obj = BrokenDel()
            del obj
            {"import gc; gc.collect()" * PYPY}

        def test_2(): pass
        """
    )
    result = pytester.runpytest()
    assert result.ret == pytest.ExitCode.TESTS_FAILED
    result.assert_outcomes(passed=1, failed=1)


@pytest.mark.filterwarnings("error::pytest.PytestUnraisableExceptionWarning")
def test_unraisable_warning_multiple_errors(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it=f"""
        class BrokenDel:
            def __init__(self, msg: str):
                self.msg = msg

            def __del__(self) -> None:
                raise ValueError(self.msg)

        def test_it() -> None:
            BrokenDel("del is broken 1")
            BrokenDel("del is broken 2")
            {"import gc; gc.collect()" * PYPY}

        def test_2(): pass
        """
    )
    result = pytester.runpytest()
    assert result.ret == pytest.ExitCode.TESTS_FAILED
    result.assert_outcomes(passed=1, failed=1)
    result.stdout.fnmatch_lines(
        [
            "  | *ExceptionGroup: multiple unraisable exception warnings (2 sub-exceptions)"
        ]
    )


def test_unraisable_collection_failure(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it=f"""
        class BrokenDel:
            def __del__(self):
                raise ValueError("del is broken")

        def test_it():
            obj = BrokenDel()
            del obj
            {"import gc; gc.collect()" * PYPY}

        def test_2(): pass
        """
    )

    class MyError(BaseException):
        pass

    with mock.patch("traceback.format_exception", side_effect=MyError):
        result = pytester.runpytest()
    assert result.ret == 1
    result.assert_outcomes(passed=1, failed=1)
    result.stdout.fnmatch_lines(
        ["E               RuntimeError: Failed to process unraisable exception"]
    )


def _set_gc_state(enabled: bool) -> bool:
    was_enabled = gc.isenabled()
    if enabled:
        gc.enable()
    else:
        gc.disable()
    return was_enabled


def test_refcycle_unraisable(pytester: Pytester) -> None:
    # see: https://github.com/pytest-dev/pytest/issues/10404
    pytester.makepyfile(
        test_it="""
        # Should catch the unraisable exception even if gc is disabled.
        import gc; gc.disable()

        import pytest

        class BrokenDel:
            def __init__(self):
                self.self = self  # make a reference cycle

            def __del__(self):
                raise ValueError("del is broken")

        def test_it():
            BrokenDel()
        """
    )

    result = pytester.runpytest_subprocess(
        "-Wdefault::pytest.PytestUnraisableExceptionWarning"
    )

    assert result.ret == 0

    result.assert_outcomes(passed=1)
    result.stderr.fnmatch_lines("ValueError: del is broken")


def test_refcycle_unraisable_warning_filter(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it="""
        # Should catch the unraisable exception even if gc is disabled.
        import gc; gc.disable()

        import pytest

        class BrokenDel:
            def __init__(self):
                self.self = self  # make a reference cycle

            def __del__(self):
                raise ValueError("del is broken")

        def test_it():
            BrokenDel()
        """
    )

    result = pytester.runpytest_subprocess(
        "-Werror::pytest.PytestUnraisableExceptionWarning"
    )

    # TODO: Should be a test failure or error. Currently the exception
    # propagates all the way to the top resulting in exit code 1.
    assert result.ret == 1

    result.assert_outcomes(passed=1)
    result.stderr.fnmatch_lines("ValueError: del is broken")


def test_create_task_raises_unraisable_warning_filter(pytester: Pytester) -> None:
    # note that the host pytest warning filter is disabled and the pytester
    # warning filter applies during config teardown of unraisablehook.
    # see: https://github.com/pytest-dev/pytest/issues/10404
    # This is a dupe of the above test, but using the exact reproducer from
    # the issue
    pytester.makepyfile(
        test_it="""
        # Should catch the unraisable exception even if gc is disabled.
        import gc; gc.disable()

        import asyncio
        import pytest

        async def my_task():
            pass

        def test_scheduler_must_be_created_within_running_loop() -> None:
            with pytest.raises(RuntimeError) as _:
                asyncio.create_task(my_task())
        """
    )

    result = pytester.runpytest_subprocess("-Werror")

    # TODO: Should be a test failure or error. Currently the exception
    # propagates all the way to the top resulting in exit code 1.
    # -Werror matches the wrapping PytestUnraisableExceptionWarning, so the
    # wrapper propagates; its message embeds the original RuntimeWarning.
    assert result.ret == 1

    result.assert_outcomes(passed=1)
    result.stderr.fnmatch_lines("RuntimeWarning: coroutine 'my_task' was never awaited")


def test_refcycle_unraisable_warning_filter_default(pytester: Pytester) -> None:
    # note this time we use a default warning filter for pytester
    # and run it in a subprocess, because the warning can only go to the
    # sys.stdout rather than the terminal reporter, which has already
    # finished.
    # see: https://github.com/pytest-dev/pytest/pull/13057#discussion_r1888396126
    pytester.makepyfile(
        test_it="""
        import gc
        gc.disable()

        import pytest

        class BrokenDel:
            def __init__(self):
                self.self = self  # make a reference cycle

            def __del__(self):
                raise ValueError("del is broken")

        def test_it():
            BrokenDel()
        """
    )

    # since we use subprocess we need to disable gc inside test_it
    result = pytester.runpytest_subprocess("-Wdefault")

    assert result.ret == pytest.ExitCode.OK

    # TODO: should be warnings=1, but the outcome has already come out
    # by the time the warning triggers
    result.assert_outcomes(passed=1)
    result.stderr.fnmatch_lines("ValueError: del is broken")


def test_unraisable_warning_without_filter_still_wraps(pytester: Pytester) -> None:
    # A Warning raised from ``__del__`` gets wrapped in
    # PytestUnraisableExceptionWarning like any other unraisable exception.
    # pytest does not unwrap it to honor a filter on the inner class, so with
    # no filter matching the wrapper, pytest logs the warning and the run
    # passes. This guards against re-introducing the dropped unwrap path.
    pytester.makepyfile(
        test_it="""
        class RaisingDel:
            def __del__(self):
                raise UserWarning("oops")

        def test_it():
            obj = RaisingDel()
            del obj
        """
    )

    # Subprocess so we don't inherit the outer pytest's ``error`` filter from
    # pyproject.toml, which would promote the wrapper and fail the run.
    # ``-Wdefault::pytest.PytestUnraisableExceptionWarning`` makes the wrapping
    # warning visible on stderr whether ``__del__`` fires inside the test or
    # during later GC (PyPy).
    result = pytester.runpytest_subprocess(
        "-Wdefault::pytest.PytestUnraisableExceptionWarning"
    )

    assert result.ret == 0
    result.assert_outcomes(passed=1)
    # The unraisable hook emitted PytestUnraisableExceptionWarning.
    # The warning lands in the warnings-summary section of stdout on
    # CPython (where ``__del__`` fires inside the test) and on stderr on
    # PyPy (where it fires during later GC). Check both rather than the
    # outcomes counter, which is timing-dependent.
    combined = "\n".join(result.outlines + result.errlines)
    assert "PytestUnraisableExceptionWarning" in combined


@pytest.mark.skipif(PYPY, reason="garbage-collection differences make this flaky")
def test_unraisable_decouples_from_cleanup_stack_order(pytester: Pytester) -> None:
    # Regression test for the structural fix. The garbage-collection step
    # that surfaces queued unraisables must run before _cleanup_stack.close()
    # so warning filters installed via cleanup-stack-managed contexts are
    # still in effect when finalizers fire. Otherwise correctness depends
    # on the order in which plugins register their cleanups under LIFO.
    #
    # The conftest uses ``@hookimpl(trylast=True)`` so its pytest_configure
    # runs after all built-in pytest_configures. Its
    # ``warnings.resetwarnings`` cleanup then lands on the cleanup stack
    # last and pops first under LIFO. Under the pre-fix layout, where
    # garbage collection runs inside unraisableexception's own cleanup
    # callback, that pop clears the user's
    # ``error::pytest.PytestUnraisableExceptionWarning`` filter before the
    # finalizer raises; pytest logs the wrapped warning instead of
    # promoting it, and the suite exits 0. Under the post-fix layout,
    # ``pytest_unconfigure`` runs the garbage collection and queue processing
    # before the cleanup stack starts closing, so the conftest's reset has no
    # effect.
    pytester.makeini(
        """
        [pytest]
        filterwarnings =
            error::pytest.PytestUnraisableExceptionWarning
        """
    )
    pytester.makeconftest(
        """
        import warnings
        import pytest

        @pytest.hookimpl(trylast=True)
        def pytest_configure(config):
            config.add_cleanup(warnings.resetwarnings)
        """
    )
    pytester.makepyfile(
        test_it="""
        import gc; gc.disable()

        class BrokenDel:
            def __init__(self):
                self.self = self  # cycle survives until session-end gc

            def __del__(self):
                raise ValueError("del is broken")

        def test_it():
            BrokenDel()
        """
    )

    result = pytester.runpytest_subprocess()

    assert result.ret == 1, (
        "wrapper filter is still installed at GC time, so the unraisable "
        "exits non-zero regardless of cleanup-stack pop order"
    )
    result.assert_outcomes(passed=1)
    result.stderr.fnmatch_lines("*ValueError: del is broken*")


def test_pytest_unconfigure_survives_failed_pytest_configure(
    pytester: Pytester,
) -> None:
    # Regression test for the guard against an unset stash key. When
    # another plugin's pytest_configure raises pytest.UsageError, pluggy
    # stops calling the remaining configure hooks; the unraisable plugin's
    # pytest_configure never runs and config.stash[unraisable_exceptions]
    # stays unset. pytest_unconfigure still fires for partially-configured
    # runs, so without a presence check the unraisable plugin's
    # pytest_unconfigure raises KeyError and pytest reports INTERNALERROR
    # instead of USAGE_ERROR.
    pytester.makeconftest(
        """
        import pytest

        def pytest_configure(config):
            raise pytest.UsageError("simulated bad config")
        """
    )
    pytester.makepyfile(test_it="def test_it(): pass")

    result = pytester.runpytest_subprocess()

    assert result.ret == pytest.ExitCode.USAGE_ERROR, (
        "the UsageError must surface as USAGE_ERROR, not a KeyError INTERNALERROR"
    )
    result.stderr.fnmatch_lines("*ERROR: simulated bad config*")
    result.stderr.no_fnmatch_line("*INTERNALERROR*")
    result.stderr.no_fnmatch_line("*KeyError*")


@pytest.mark.filterwarnings("error::pytest.PytestUnraisableExceptionWarning")
def test_possibly_none_excinfo(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_it="""
        import sys
        import types

        def test_it():
            sys.unraisablehook(
                types.SimpleNamespace(
                    exc_type=RuntimeError,
                    exc_value=None,
                    exc_traceback=None,
                    err_msg=None,
                    object=None,
                )
            )
        """
    )

    result = pytester.runpytest()

    # TODO: should be a test failure or error
    assert result.ret == pytest.ExitCode.TESTS_FAILED

    result.assert_outcomes(failed=1)
    result.stdout.fnmatch_lines(
        [
            "E                   pytest.PytestUnraisableExceptionWarning:"
            " Exception ignored in: None",
            "E                   ",
            "E                   NoneType: None",
        ]
    )



from __future__ import annotations

import pytest


@pytest.fixture(scope="module", params=range(966))
def foo(request):
    return request.param


def test_it(foo):
    pass


def test_it2(foo):
    pass



from __future__ import annotations

from collections.abc import Iterable
from collections.abc import Iterator
from collections.abc import Sequence

from _pytest._io.pprint import PrettyPrinter
from _pytest._io.saferepr import saferepr
from _pytest.assertion._typing import _HighlightFunc
from _pytest.compat import running_on_ci


def _compare_eq_iterable(
    left: Iterable[object],
    right: Iterable[object],
    highlighter: _HighlightFunc,
    verbose: int = 0,
) -> Iterator[str]:
    if verbose <= 0 and not running_on_ci():
        yield "Use -v to get more diff"
        return
    # dynamic import to speedup pytest
    import difflib

    left_formatting = PrettyPrinter().pformat(left).splitlines()
    right_formatting = PrettyPrinter().pformat(right).splitlines()

    yield ""
    yield "Full diff:"
    # "right" is the expected base against which we compare "left",
    # see https://github.com/pytest-dev/pytest/issues/3333
    yield from highlighter(
        "\n".join(
            line.rstrip() for line in difflib.ndiff(right_formatting, left_formatting)
        ),
        lexer="diff",
    ).splitlines()


def _compare_eq_sequence(
    left: Sequence[object],
    right: Sequence[object],
    highlighter: _HighlightFunc,
    verbose: int = 0,
) -> Iterator[str]:
    comparing_bytes = isinstance(left, bytes) and isinstance(right, bytes)
    len_left = len(left)
    len_right = len(right)
    for i in range(min(len_left, len_right)):
        if left[i] != right[i]:
            if comparing_bytes:
                # when comparing bytes, we want to see their ascii representation
                # instead of their numeric values (#5260)
                # using a slice gives us the ascii representation:
                # >>> s = b'foo'
                # >>> s[0]
                # 102
                # >>> s[0:1]
                # b'f'
                left_value: object = left[i : i + 1]
                right_value: object = right[i : i + 1]
            else:
                left_value = left[i]
                right_value = right[i]

            yield (
                f"At index {i} diff:"
                f" {highlighter(repr(left_value))} != {highlighter(repr(right_value))}"
            )
            break

    if comparing_bytes:
        # when comparing bytes, it doesn't help to show the "sides contain one or more
        # items" longer explanation, so skip it
        return

    len_diff = len_left - len_right
    if len_diff:
        if len_diff > 0:
            dir_with_more = "Left"
            extra = saferepr(left[len_right])
        else:
            len_diff = 0 - len_diff
            dir_with_more = "Right"
            extra = saferepr(right[len_left])

        if len_diff == 1:
            yield f"{dir_with_more} contains one more item: {highlighter(extra)}"
        else:
            yield f"{dir_with_more} contains {len_diff} more items, first extra item: {highlighter(extra)}"



# mypy: allow-untyped-defs
from __future__ import annotations

import enum
from functools import cached_property
from functools import partial
from functools import wraps
from typing import Literal
import warnings

from _pytest.compat import assert_never
from _pytest.compat import deprecated
from _pytest.compat import get_real_func
from _pytest.compat import safe_getattr
from _pytest.compat import safe_isclass
from _pytest.outcomes import OutcomeException
import pytest


def test_real_func_loop_limit() -> None:
    class Evil:
        def __init__(self):
            self.left = 1000

        def __repr__(self):
            return f"<Evil left={self.left}>"

        def __getattr__(self, attr):
            if not self.left:
                raise RuntimeError("it's over")  # pragma: no cover
            self.left -= 1
            return self

    evil = Evil()

    with pytest.raises(
        ValueError,
        match=("wrapper loop when unwrapping <Evil left=998>"),
    ):
        get_real_func(evil)


def test_get_real_func() -> None:
    """Check that get_real_func correctly unwraps decorators until reaching the real function"""

    def decorator(f):
        @wraps(f)
        def inner():
            pass  # pragma: no cover

        return inner

    def func():
        pass  # pragma: no cover

    wrapped_func = decorator(decorator(func))
    assert get_real_func(wrapped_func) is func

    wrapped_func2 = decorator(decorator(wrapped_func))
    assert get_real_func(wrapped_func2) is func

    # obtain the function up until the point a function was wrapped by pytest itself
    @pytest.fixture
    def wrapped_func3():
        pass  # pragma: no cover

    wrapped_func4 = decorator(wrapped_func3)
    assert get_real_func(wrapped_func4) is wrapped_func3._get_wrapped_function()


def test_get_real_func_partial() -> None:
    """Test get_real_func handles partial instances correctly"""

    def foo(x):
        return x

    assert get_real_func(foo) is foo
    assert get_real_func(partial(foo)) is foo


class ErrorsHelper:
    @property
    def raise_baseexception(self):
        raise BaseException("base exception should be raised")

    @property
    def raise_exception(self):
        raise Exception("exception should be caught")

    @property
    def raise_fail_outcome(self):
        pytest.fail("fail should be caught")


def test_helper_failures() -> None:
    helper = ErrorsHelper()
    with pytest.raises(Exception):  # noqa: B017
        _ = helper.raise_exception
    with pytest.raises(OutcomeException):
        _ = helper.raise_fail_outcome


def test_safe_getattr() -> None:
    helper = ErrorsHelper()
    assert safe_getattr(helper, "raise_exception", "default") == "default"
    assert safe_getattr(helper, "raise_fail_outcome", "default") == "default"
    with pytest.raises(BaseException):  # noqa: B017
        assert safe_getattr(helper, "raise_baseexception", "default")


def test_safe_isclass() -> None:
    assert safe_isclass(type) is True

    class CrappyClass(Exception):
        # Type ignored because it's bypassed intentionally.
        @property  # type: ignore
        def __class__(self):
            assert False, "Should be ignored"

    assert safe_isclass(CrappyClass()) is False


def test_cached_property() -> None:
    ncalls = 0

    class Class:
        @cached_property
        def prop(self) -> int:
            nonlocal ncalls
            ncalls += 1
            return ncalls

    c1 = Class()
    assert ncalls == 0
    assert c1.prop == 1
    assert c1.prop == 1
    c2 = Class()
    assert ncalls == 1
    assert c2.prop == 2
    assert c1.prop == 1


def test_assert_never_union() -> None:
    x: int | str = 10

    if isinstance(x, int):
        pass
    else:
        with pytest.raises(AssertionError):
            assert_never(x)  # type: ignore[arg-type]

    if isinstance(x, int):
        pass
    elif isinstance(x, str):
        pass
    else:
        assert_never(x)


def test_assert_never_enum() -> None:
    E = enum.Enum("E", "a b")
    x: E = E.a

    if x is E.a:
        pass
    else:
        with pytest.raises(AssertionError):
            assert_never(x)  # type: ignore[arg-type]

    if x is E.a:
        pass
    elif x is E.b:
        pass
    else:
        assert_never(x)


def test_assert_never_literal() -> None:
    x: Literal["a", "b"] = "a"

    if x == "a":
        pass
    else:
        with pytest.raises(AssertionError):
            assert_never(x)  # type: ignore[arg-type]

    if x == "a":
        pass
    elif x == "b":
        pass
    else:
        assert_never(x)


def test_deprecated() -> None:
    # This test is mostly for coverage.

    @deprecated("This is deprecated!")
    def old_way() -> str:
        return "human intelligence"

    with warnings.catch_warnings():
        warnings.simplefilter("ignore", DeprecationWarning)
        assert old_way() == "human intelligence"  # type: ignore[deprecated]



from __future__ import annotations

import collections.abc
from collections.abc import Mapping
import dataclasses
from typing import TypeGuard


def issequence(x: object) -> TypeGuard[collections.abc.Sequence[object]]:
    return isinstance(x, collections.abc.Sequence) and not isinstance(x, str)


def istext(x: object) -> TypeGuard[str]:
    return isinstance(x, str)


def ismapping(x: object) -> TypeGuard[Mapping[object, object]]:
    return isinstance(x, Mapping)


def isset(x: object) -> TypeGuard[set[object] | frozenset[object]]:
    return isinstance(x, set | frozenset)


def isnamedtuple(obj: object) -> bool:
    return isinstance(obj, tuple) and getattr(obj, "_fields", None) is not None


isdatacls = dataclasses.is_dataclass


def isattrs(obj: object) -> bool:
    return getattr(obj, "__attrs_attrs__", None) is not None


def isiterable(obj: object) -> TypeGuard[collections.abc.Iterable[object]]:
    try:
        iter(obj)  # type: ignore[call-overload]
        return not istext(obj)
    except Exception:
        return False


def has_default_eq(obj: object) -> bool:
    """Check if an instance of an object contains the default eq

    First, we check if the object's __eq__ attribute has __code__,
    if so, we check the equally of the method code filename (__code__.co_filename)
    to the default one generated by the dataclass and attr module
    for dataclasses the default co_filename is <string>, for attrs class, the __eq__ should contain "attrs eq generated"
    """
    # inspired from https://github.com/willmcgugan/rich/blob/07d51ffc1aee6f16bd2e5a25b4e82850fb9ed778/rich/pretty.py#L68
    if hasattr(obj.__eq__, "__code__") and hasattr(obj.__eq__.__code__, "co_filename"):
        code_filename = obj.__eq__.__code__.co_filename

        if isattrs(obj):
            return "attrs generated " in code_filename

        return code_filename == "<string>"  # data class
    return True



from __future__ import annotations

from typing import Literal


def dummy_highlighter(source: str, lexer: Literal["diff", "python"] = "python") -> str:
    """Dummy highlighter that returns the text unprocessed.

    Needed for _notin_text, as the diff gets post-processed to only show the "+" part.
    """
    return source



"""Generate an executable with pytest runner embedded using PyInstaller."""

from __future__ import annotations


if __name__ == "__main__":
    import subprocess

    import pytest

    hidden = []
    for x in pytest.freeze_includes():
        hidden.extend(["--hidden-import", x])
    hidden.extend(["--hidden-import", "distutils"])
    args = ["pyinstaller", "--noconfirm", *hidden, "runtests_script.py"]
    subprocess.check_call(" ".join(args), shell=True)



# mypy: allow-untyped-defs
from __future__ import annotations

from _pytest.fixtures import FixtureLookupError
import pytest


@pytest.fixture
def arg2(request):
    with pytest.raises(FixtureLookupError):
        request.getfixturevalue("arg1")



from __future__ import annotations

from collections.abc import Iterator
import dataclasses
import pprint

from _pytest.assertion._compare_mapping import _compare_eq_mapping
from _pytest.assertion._compare_sequence import _compare_eq_iterable
from _pytest.assertion._compare_sequence import _compare_eq_sequence
from _pytest.assertion._compare_set import _compare_eq_set
from _pytest.assertion._guards import has_default_eq
from _pytest.assertion._guards import isattrs
from _pytest.assertion._guards import isdatacls
from _pytest.assertion._guards import isiterable
from _pytest.assertion._guards import ismapping
from _pytest.assertion._guards import isnamedtuple
from _pytest.assertion._guards import issequence
from _pytest.assertion._guards import isset
from _pytest.assertion._guards import istext
from _pytest.assertion._typing import _AssertionTextDiffStyle
from _pytest.assertion._typing import _HighlightFunc
from _pytest.assertion.compare_text import _compare_eq_text


def _compare_eq_any(
    left: object,
    right: object,
    highlighter: _HighlightFunc,
    verbose: int,
    assertion_text_diff_style: _AssertionTextDiffStyle,
) -> Iterator[str]:
    """Yield the per-line explanation for ``left == right`` (without summary).

    Yields nothing when no specialised explanation applies, so consumers
    can stream the output and bail out early (e.g. for truncation) without
    materialising the entire diff first.
    """
    if istext(left) and istext(right):
        yield from _compare_eq_text(
            left,
            right,
            highlighter,
            verbose,
            assertion_text_diff_style,
        )
    else:
        from _pytest.python_api import ApproxBase

        # Although the common order should be obtained == approx(...), allow both ways.
        if isinstance(right, ApproxBase):
            yield from right._repr_compare(left)
        elif isinstance(left, ApproxBase):
            yield from left._repr_compare(right)
        elif type(left) is type(right) and (
            isdatacls(left) or isattrs(left) or isnamedtuple(left)
        ):
            # Note: unlike dataclasses/attrs, namedtuples compare only the
            # field values, not the type or field names. But this branch
            # intentionally only handles the same-type case, which was often
            # used in older code bases before dataclasses/attrs were available.
            yield from _compare_eq_cls(
                left,
                right,
                highlighter,
                verbose,
                assertion_text_diff_style,
            )
        elif issequence(left) and issequence(right):
            yield from _compare_eq_sequence(left, right, highlighter, verbose)
        elif isset(left) and isset(right):
            yield from _compare_eq_set(left, right, highlighter, verbose)
        elif ismapping(left) and ismapping(right):
            yield from _compare_eq_mapping(left, right, highlighter, verbose)

        if isiterable(left) and isiterable(right):
            yield from _compare_eq_iterable(left, right, highlighter, verbose)


def _compare_eq_cls(
    left: object,
    right: object,
    highlighter: _HighlightFunc,
    verbose: int,
    assertion_text_diff_style: _AssertionTextDiffStyle,
) -> Iterator[str]:
    if not has_default_eq(left):
        return
    if isdatacls(left):
        all_fields = dataclasses.fields(left)
        fields_to_check = [info.name for info in all_fields if info.compare]
    elif isattrs(left):
        all_fields = left.__attrs_attrs__  # type: ignore[attr-defined]
        fields_to_check = [field.name for field in all_fields if getattr(field, "eq")]
    elif isnamedtuple(left):
        fields_to_check = left._fields  # type: ignore[attr-defined]
    else:
        assert False

    indent = "  "
    same = []
    diff = []
    for field in fields_to_check:
        if getattr(left, field) == getattr(right, field):
            same.append(field)
        else:
            diff.append(field)

    if same or diff:
        yield ""
    if same and verbose < 2:
        yield f"Omitting {len(same)} identical items, use -vv to show"
    elif same:
        yield "Matching attributes:"
        yield from highlighter(pprint.pformat(same)).splitlines()
    if diff:
        yield "Differing attributes:"
        yield from highlighter(pprint.pformat(diff)).splitlines()
        for field in diff:
            field_left = getattr(left, field)
            field_right = getattr(right, field)
            yield ""
            yield f"Drill down into differing attribute {field}:"
            yield f"{indent}{field}: {highlighter(repr(field_left))} != {highlighter(repr(field_right))}"
            for line in _compare_eq_any(
                field_left,
                field_right,
                highlighter,
                verbose,
                assertion_text_diff_style,
            ):
                yield indent + line



# mypy: allow-untyped-defs
from __future__ import annotations

from _pytest import deprecated
from _pytest.pytester import Pytester
import pytest
from pytest import PytestDeprecationWarning


@pytest.mark.parametrize("plugin", sorted(deprecated.DEPRECATED_EXTERNAL_PLUGINS))
@pytest.mark.filterwarnings("default")
def test_external_plugins_integrated(pytester: Pytester, plugin) -> None:
    pytester.syspathinsert()
    pytester.makepyfile(**{plugin: ""})

    with pytest.warns(pytest.PytestConfigWarning):
        pytester.parseconfig("-p", plugin)


def test_hookspec_via_function_attributes_are_deprecated():
    from _pytest.config import PytestPluginManager

    pm = PytestPluginManager()

    class DeprecatedHookMarkerSpec:
        def pytest_bad_hook(self):
            pass

        pytest_bad_hook.historic = False  # type: ignore[attr-defined]

    with pytest.warns(
        PytestDeprecationWarning,
        match=r"Please use the pytest\.hookspec\(historic=False\) decorator",
    ) as recorder:
        pm.add_hookspecs(DeprecatedHookMarkerSpec)
    (record,) = recorder
    assert (
        record.lineno
        == DeprecatedHookMarkerSpec.pytest_bad_hook.__code__.co_firstlineno
    )
    assert record.filename == __file__


def test_hookimpl_via_function_attributes_are_deprecated():
    from _pytest.config import PytestPluginManager

    pm = PytestPluginManager()

    class DeprecatedMarkImplPlugin:
        def pytest_runtest_call(self):
            pass

        pytest_runtest_call.tryfirst = True  # type: ignore[attr-defined]

    with pytest.warns(
        PytestDeprecationWarning,
        match=r"Please use the pytest.hookimpl\(tryfirst=True\)",
    ) as recorder:
        pm.register(DeprecatedMarkImplPlugin())
    (record,) = recorder
    assert (
        record.lineno
        == DeprecatedMarkImplPlugin.pytest_runtest_call.__code__.co_firstlineno
    )
    assert record.filename == __file__


def test_yield_fixture_is_deprecated() -> None:
    with pytest.warns(DeprecationWarning, match=r"yield_fixture is deprecated"):

        @pytest.yield_fixture  # type: ignore[deprecated]
        def fix():
            assert False


def test_private_is_deprecated() -> None:
    class PrivateInit:
        def __init__(self, foo: int, *, _ispytest: bool = False) -> None:
            deprecated.check_ispytest(_ispytest)

    with pytest.warns(
        pytest.PytestDeprecationWarning, match="private pytest class or function"
    ):
        PrivateInit(10)

    # Doesn't warn.
    PrivateInit(10, _ispytest=True)


def test_class_scope_instance_method_is_deprecated(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import pytest

        class TestClass:
            @pytest.fixture(scope="class")
            def fix(self):
                self.attr = True

            def test_foo(self, fix):
                pass
        """
    )
    result = pytester.runpytest("-Werror::pytest.PytestRemovedIn10Warning")
    result.assert_outcomes(errors=1)
    result.stdout.fnmatch_lines(
        ["*PytestRemovedIn10Warning: Class-scoped fixture defined as instance method*"]
    )


def test_class_scope_classmethod_fixture_not_deprecated(pytester: Pytester) -> None:
    """A class-scoped fixture defined as @classmethod does NOT warn."""
    pytester.makepyfile(
        """
        import pytest

        class TestClass:
            @pytest.fixture(scope="class")
            @classmethod
            def fix(cls):
                cls.attr = True

            def test_foo(self, fix):
                assert type(self).attr is True
        """
    )
    result = pytester.runpytest("-Werror::pytest.PytestRemovedIn10Warning")
    result.assert_outcomes(passed=1)


class TestFixtureNodeidDeprecations:
    """Tests for deprecated baseid/nodeid string APIs in fixture registration.

    AI-generated coverage tests for legacy paths that will be removed in
    pytest 10. These exist solely to maintain patch coverage until the
    deprecated code is deleted.

    Legacy paths covered:
    - parsefactories(obj, nodeid_string) deprecation warning
    - parsefactories(obj, None) does NOT warn (standard plugin pattern)
    - parsefactories() with no args raises TypeError
    - _register_fixture(nodeid=string) deprecation warning
    - _nodeid_autousenames population and _getautousenames yield
    - _matchfactories string-based fallback (match + non-match branches)
    """

    def test_parsefactories_nodeid_deprecation(self, pytester: Pytester) -> None:
        """parsefactories(obj, "path") warns; parsefactories(obj, None) does not."""
        pytester.makeconftest(
            """
            import pytest
            import types
            import warnings

            def pytest_collection_modifyitems(session, items):
                fm = session._fixturemanager

                @pytest.fixture
                def fix_a():
                    return "a"

                @pytest.fixture
                def fix_b():
                    return "b"

                mod_with_path = types.ModuleType("plugin_path")
                mod_with_path.fix_a = fix_a
                mod_none = types.ModuleType("plugin_none")
                mod_none.fix_b = fix_b

                with warnings.catch_warnings(record=True) as w:
                    warnings.simplefilter("always")
                    fm.parsefactories(mod_with_path, "some/nodeid")
                    fm.parsefactories(mod_none, None)

                nodeid_warns = [x for x in w if "parsefactories" in str(x.message)]
                assert len(nodeid_warns) == 2, f"Expected 2 warning, got: {w}"
            """
        )
        pytester.makepyfile(
            """
            def test_global_fix(fix_b):
                assert fix_b == "b"
            """
        )
        result = pytester.runpytest("-W", "default::pytest.PytestRemovedIn10Warning")
        result.assert_outcomes(passed=1)

    def test_parsefactories_no_args_raises_typeerror(self, pytester: Pytester) -> None:
        """parsefactories() with no holder and no node_or_obj raises TypeError."""
        pytester.makeconftest(
            """
            import pytest

            def pytest_collection_modifyitems(session, items):
                fm = session._fixturemanager
                with pytest.raises(TypeError, match="requires holder or node_or_obj"):
                    fm.parsefactories()
            """
        )
        pytester.makepyfile("def test_pass(): pass")
        result = pytester.runpytest()
        result.assert_outcomes(passed=1)

    def test_register_fixture_nodeid_and_autouse_legacy(
        self, pytester: Pytester
    ) -> None:
        """_register_fixture(nodeid=string) warns and autouse populates/yields.

        Covers end-to-end:
        - Deprecation warning on _register_fixture(nodeid=...)
        - _nodeid_autousenames populated for autouse + non-empty nodeid
        - _getautousenames yields from nodeid_basenames at lookup time
        """
        pytester.makeconftest(
            """
            import pytest
            import warnings

            _done = False

            def pytest_collectstart(collector):
                global _done
                if _done or not hasattr(collector.session, "_fixturemanager"):
                    return
                if collector.nodeid == "":
                    return
                _done = True
                fm = collector.session._fixturemanager
                with warnings.catch_warnings(record=True) as w:
                    warnings.simplefilter("always")
                    fm._register_fixture(
                        name="legacy_autouse",
                        func=lambda: None,
                        nodeid=collector.nodeid,
                        autouse=True,
                    )
                assert any("_register_fixture" in str(x.message) for x in w)
                assert "legacy_autouse" in fm._nodeid_autousenames[collector.nodeid]
            """
        )
        pytester.makepyfile(
            """
            def test_autouse_yielded(request):
                assert "legacy_autouse" in request.fixturenames
            """
        )
        result = pytester.runpytest("-W", "default::pytest.PytestRemovedIn10Warning")
        result.assert_outcomes(passed=1)

    def test_matchfactories_string_fallback(self, pytester: Pytester) -> None:
        """_matchfactories uses baseid string matching for legacy fixtures.

        Exercises both branches:
        - baseid="" matches all nodes (global fixture)
        - baseid="nonexistent/path" matches nothing (scoped fixture invisible)
        """
        pytester.makeconftest(
            """
            import pytest
            import warnings

            def pytest_collection_modifyitems(session, items):
                fm = session._fixturemanager
                with warnings.catch_warnings():
                    warnings.simplefilter("ignore")
                    fm._register_fixture(
                        name="global_legacy", func=lambda: "ok", nodeid=""
                    )
                    fm._register_fixture(
                        name="scoped_legacy", func=lambda: "nope",
                        nodeid="nonexistent/path",
                    )
            """
        )
        pytester.makepyfile(
            """
            def test_global_visible(global_legacy):
                assert global_legacy == "ok"

            def test_scoped_invisible(request):
                defs = request.session._fixturemanager.getfixturedefs(
                    "scoped_legacy", request._pyfuncitem
                )
                assert defs == ()
            """
        )
        result = pytester.runpytest("-W", "ignore::pytest.PytestRemovedIn10Warning")
        result.assert_outcomes(passed=2)

    def test_fixturedef_has_location_deprecated(self, pytester: Pytester) -> None:
        """Accessing FixtureDef.has_location warns."""
        pytester.makepyfile(
            """
            import pytest

            @pytest.fixture
            def fix():
                return 1

            def test_it(request):
                fixturedef = request._fixturemanager.getfixturedefs(
                    "fix", request._pyfuncitem
                )[0]
                with pytest.warns(
                    pytest.PytestRemovedIn10Warning, match="has_location"
                ):
                    assert fixturedef.has_location is True
            """
        )
        result = pytester.runpytest()
        result.assert_outcomes(passed=1)



from __future__ import annotations

from _pytest.pytester import Pytester


def test_should_show_no_output_when_zero_items(pytester: Pytester) -> None:
    result = pytester.runpytest("--fixtures-per-test")
    result.stdout.no_fnmatch_line("*fixtures used by*")
    assert result.ret == 0


def test_fixtures_in_module(pytester: Pytester) -> None:
    p = pytester.makepyfile(
        '''
        import pytest
        @pytest.fixture
        def _arg0():
            """hidden arg0 fixture"""
        @pytest.fixture
        def arg1():
            """arg1 docstring"""
        def test_arg1(arg1):
            pass
    '''
    )

    result = pytester.runpytest("--fixtures-per-test", p)
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_arg1*",
            "*(test_fixtures_in_module.py:9)*",
            "arg1 -- test_fixtures_in_module.py:6",
            "    arg1 docstring",
        ]
    )
    result.stdout.no_fnmatch_line("*_arg0*")


def test_fixtures_in_conftest(pytester: Pytester) -> None:
    pytester.makeconftest(
        '''
        import pytest
        @pytest.fixture
        def arg1():
            """arg1 docstring"""
        @pytest.fixture
        def arg2():
            """arg2 docstring"""
        @pytest.fixture
        def arg3(arg1, arg2):
            """arg3
            docstring
            """
    '''
    )
    p = pytester.makepyfile(
        """
        def test_arg2(arg2):
            pass
        def test_arg3(arg3):
            pass
    """
    )
    result = pytester.runpytest("--fixtures-per-test", p)
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_arg2*",
            "*(test_fixtures_in_conftest.py:2)*",
            "arg2 -- conftest.py:6",
            "    arg2 docstring",
            "*fixtures used by test_arg3*",
            "*(test_fixtures_in_conftest.py:4)*",
            "arg1 -- conftest.py:3",
            "    arg1 docstring",
            "arg2 -- conftest.py:6",
            "    arg2 docstring",
            "arg3 -- conftest.py:9",
            "    arg3",
        ]
    )


def test_should_show_fixtures_used_by_test(pytester: Pytester) -> None:
    pytester.makeconftest(
        '''
        import pytest
        @pytest.fixture
        def arg1():
            """arg1 from conftest"""
        @pytest.fixture
        def arg2():
            """arg2 from conftest"""
    '''
    )
    p = pytester.makepyfile(
        '''
        import pytest
        @pytest.fixture
        def arg1():
            """arg1 from testmodule"""
        def test_args(arg1, arg2):
            pass
    '''
    )
    result = pytester.runpytest("--fixtures-per-test", p)
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_args*",
            "*(test_should_show_fixtures_used_by_test.py:6)*",
            "arg1 -- test_should_show_fixtures_used_by_test.py:3",
            "    arg1 from testmodule",
            "arg2 -- conftest.py:6",
            "    arg2 from conftest",
        ]
    )


def test_verbose_include_private_fixtures_and_loc(pytester: Pytester) -> None:
    pytester.makeconftest(
        '''
        import pytest
        @pytest.fixture
        def _arg1():
            """_arg1 from conftest"""
        @pytest.fixture
        def arg2(_arg1):
            """arg2 from conftest"""
    '''
    )
    p = pytester.makepyfile(
        '''
        import pytest
        @pytest.fixture
        def arg3():
            """arg3 from testmodule"""
        def test_args(arg2, arg3):
            pass
    '''
    )
    result = pytester.runpytest("--fixtures-per-test", "-v", p)
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_args*",
            "*(test_verbose_include_private_fixtures_and_loc.py:6)*",
            "_arg1 -- conftest.py:3",
            "    _arg1 from conftest",
            "arg2 -- conftest.py:6",
            "    arg2 from conftest",
            "arg3 -- test_verbose_include_private_fixtures_and_loc.py:3",
            "    arg3 from testmodule",
        ]
    )


def test_doctest_items(pytester: Pytester) -> None:
    pytester.makepyfile(
        '''
        def foo():
            """
            >>> 1 + 1
            2
            """
    '''
    )
    pytester.maketxtfile(
        """
        >>> 1 + 1
        2
    """
    )
    result = pytester.runpytest(
        "--fixtures-per-test", "--doctest-modules", "--doctest-glob=*.txt", "-v"
    )
    assert result.ret == 0

    result.stdout.fnmatch_lines(["*collected 2 items*"])


def test_multiline_docstring_in_module(pytester: Pytester) -> None:
    p = pytester.makepyfile(
        '''
        import pytest
        @pytest.fixture
        def arg1():
            """Docstring content that spans across multiple lines,
            through second line,
            and through third line.

            Docstring content that extends into a second paragraph.

            Docstring content that extends into a third paragraph.
            """
        def test_arg1(arg1):
            pass
    '''
    )

    result = pytester.runpytest("--fixtures-per-test", p)
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_arg1*",
            "*(test_multiline_docstring_in_module.py:13)*",
            "arg1 -- test_multiline_docstring_in_module.py:3",
            "    Docstring content that spans across multiple lines,",
            "    through second line,",
            "    and through third line.",
        ]
    )


def test_verbose_include_multiline_docstring(pytester: Pytester) -> None:
    p = pytester.makepyfile(
        '''
        import pytest
        @pytest.fixture
        def arg1():
            """Docstring content that spans across multiple lines,
            through second line,
            and through third line.

            Docstring content that extends into a second paragraph.

            Docstring content that extends into a third paragraph.
            """
        def test_arg1(arg1):
            pass
    '''
    )

    result = pytester.runpytest("--fixtures-per-test", "-v", p)
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_arg1*",
            "*(test_verbose_include_multiline_docstring.py:13)*",
            "arg1 -- test_verbose_include_multiline_docstring.py:3",
            "    Docstring content that spans across multiple lines,",
            "    through second line,",
            "    and through third line.",
            "    ",
            "    Docstring content that extends into a second paragraph.",
            "    ",
            "    Docstring content that extends into a third paragraph.",
        ]
    )


def test_should_not_show_direct_param_fixtures(pytester: Pytester) -> None:
    """A direct-param fixture is a helper fixture created as an implementation
    detail of direct parametrization.

    These fixtures should not be included in the output because they don't
    satisfy user expectations for how fixtures are created and used (#11295).
    """
    pytester.makepyfile(
        """
        import pytest

        @pytest.mark.parametrize("x", [1])
        def test_pseudo_fixture(x):
            pass
        """
    )
    result = pytester.runpytest("--fixtures-per-test")
    result.stdout.no_fnmatch_line("*fixtures used by*")
    assert result.ret == 0


def test_should_show_parametrized_fixtures_used_by_test(pytester: Pytester) -> None:
    """A fixture with parameters should be included if it was created using
    the @pytest.fixture decorator, including those that are indirectly
    parametrized."""
    pytester.makepyfile(
        '''
        import pytest

        @pytest.fixture(params=['a', 'b'])
        def directly(request):
            """parametrized fixture"""
            return request.param

        @pytest.fixture
        def indirectly(request):
            """indirectly parametrized fixture"""
            return request.param

        def test_directly_parametrized_fixture(directly):
            pass

        @pytest.mark.parametrize("indirectly", ["a", "b"], indirect=True)
        def test_indirectly_parametrized_fixture(indirectly):
            pass
        '''
    )
    result = pytester.runpytest("--fixtures-per-test")
    assert result.ret == 0

    result.stdout.fnmatch_lines(
        [
            "*fixtures used by test_directly_parametrized_fixture*",
            "*(test_should_show_parametrized_fixtures_used_by_test.py:14)*",
            "directly -- test_should_show_parametrized_fixtures_used_by_test.py:4",
            "    parametrized fixture",
            "*fixtures used by test_directly_parametrized_fixture*",
            "*(test_should_show_parametrized_fixtures_used_by_test.py:14)*",
            "directly -- test_should_show_parametrized_fixtures_used_by_test.py:4",
            "    parametrized fixture",
            "*fixtures used by test_indirectly_parametrized_fixture*",
            "*(test_should_show_parametrized_fixtures_used_by_test.py:17)*",
            "indirectly -- test_should_show_parametrized_fixtures_used_by_test.py:9",
            "    indirectly parametrized fixture",
            "*fixtures used by test_indirectly_parametrized_fixture*",
            "*(test_should_show_parametrized_fixtures_used_by_test.py:17)*",
            "indirectly -- test_should_show_parametrized_fixtures_used_by_test.py:9",
            "    indirectly parametrized fixture",
        ]
    )



"""Helper plugin for pytester; should not be loaded on its own."""

# This plugin contains assertions used by pytester. pytester cannot
# contain them itself, since it is imported by the `pytest` module,
# hence cannot be subject to assertion rewriting, which requires a
# module to not be already imported.
from __future__ import annotations

from collections.abc import Sequence

from _pytest.reports import CollectReport
from _pytest.reports import TestReport


def assertoutcome(
    outcomes: tuple[
        Sequence[TestReport],
        Sequence[CollectReport | TestReport],
        Sequence[CollectReport | TestReport],
    ],
    passed: int = 0,
    skipped: int = 0,
    failed: int = 0,
) -> None:
    __tracebackhide__ = True

    realpassed, realskipped, realfailed = outcomes
    obtained = {
        "failed": len(realfailed),
        "passed": len(realpassed),
        "skipped": len(realskipped),
    }
    expected = {"failed": failed, "passed": passed, "skipped": skipped}
    assert obtained == expected, outcomes


def assert_outcomes(
    outcomes: dict[str, int],
    passed: int = 0,
    skipped: int = 0,
    failed: int = 0,
    errors: int = 0,
    xpassed: int = 0,
    xfailed: int = 0,
    warnings: int | None = None,
    deselected: int | None = None,
) -> None:
    """Assert that the specified outcomes appear with the respective
    numbers (0 means it didn't occur) in the text output from a test run."""
    __tracebackhide__ = True

    obtained = {
        "passed": outcomes.get("passed", 0),
        "skipped": outcomes.get("skipped", 0),
        "failed": outcomes.get("failed", 0),
        "errors": outcomes.get("errors", 0),
        "xpassed": outcomes.get("xpassed", 0),
        "xfailed": outcomes.get("xfailed", 0),
    }
    expected = {
        "passed": passed,
        "skipped": skipped,
        "failed": failed,
        "errors": errors,
        "xpassed": xpassed,
        "xfailed": xfailed,
    }
    if warnings is not None:
        obtained["warnings"] = outcomes.get("warnings", 0)
        expected["warnings"] = warnings
    if deselected is not None:
        obtained["deselected"] = outcomes.get("deselected", 0)
        expected["deselected"] = deselected
    assert obtained == expected
